from datetime import date, datetime
from django.shortcuts import render

from qr_code.qrcode.utils import (
    MeCard,
    VCard,
    WifiConfig,
    Coordinates,
    QRCodeOptions,
    Email,
    EpcData,
    VEvent,
    EventClass,
    EventStatus,
    EventTransparency,
)

# Use a ContactDetail instance to encapsulate the detail of the contact.
DEMO_MECARD_CONTACT = MeCard(
    name="Doe, John",
    phone="+41769998877",
    email="j.doe@company.com",
    url="http://www.company.com",
    birthday=date(year=1985, month=10, day=2),
    memo="Development Manager",
    org="Company Ltd",
)

DEMO_VCARD_CONTACT = VCard(
    name="Doe; John",
    phone="+41769998877",
    email="j.doe@company.com",
    url="http://www.company.com",
    birthday=date(year=1985, month=10, day=2),
    street="Cras des Fourches 987",
    city="Delémont",
    zipcode=2800,
    region="Jura",
    country="Switzerland",
    memo="Development Manager",
    org="Company Ltd",
)

DEMO_EVENT = VEvent(
    uid="some-event-id",
    summary="Vacations",
    start=datetime(2022, 8, 6, hour=8, minute=30),
    end=datetime(2022, 8, 17, hour=12),
    location="New-York, Statue de la Liberté",
    geo=(40.69216097988203, -74.04460073403436),
    categories=["PERSO", "holidays"],
    status=EventStatus.CONFIRMED,
    event_class=EventClass.PRIVATE,
    transparency=EventTransparency.OPAQUE,
    organizer="foo@bar.com",
    url="https://bar.com",
    description="""Fake description. Meeting to provide technical review for "Phoenix" design. Happy Face Conference Room.

Phoenix design team MUST attend this meeting.

RSVP to team leader.""",
)

# Use a WifiConfig instance to encapsulate the configuration of the connexion.
DEMO_WIFI = WifiConfig(ssid="my-wifi", authentication=WifiConfig.AUTHENTICATION.WPA, password="wifi-password")

DEMO_COORDINATES = Coordinates(latitude=586000.32, longitude=250954.19, altitude=500)

DEMO_OPTIONS = QRCodeOptions(size="t", border=6, error_correction="L")


def index(request):
    """
    Build the home page of this demo app.

    :param request:
    :return: HTTP response providing the home page of this demo app.
    """

    # Build context for rendering QR codes.
    context = dict(
        mecard_contact=DEMO_MECARD_CONTACT,
        vcard_contact=DEMO_VCARD_CONTACT,
        wifi_config=DEMO_WIFI,
        video_id="J9go2nj6b3M",
        google_maps_coordinates=DEMO_COORDINATES,
        geolocation_coordinates=DEMO_COORDINATES,
        email=Email(
            to="john.doe@domain.com",
            cc=("bob.doe@domain.com", "alice.doe@domain.com"),
            bcc="secret@domain.com",
            subject="Important message",
            body="This is a very important message!",
        ),
        epc_data=EpcData(
            name="Wikimedia Foerdergesellschaft", iban="DE33100205000001194700", amount=20, text="To Wikipedia, From Gérard Boéchat"
        ),
        event=DEMO_EVENT,
        shift_js_encoded="ウェブサイトにおける文字コードの割合、UTF-8が90％超え。Shift_JISやEUC-JPは？".encode("shift-jis"),
        kanji_encoded="義務教育諸学校教科用図書検定基準".encode("cp932"),
        options_example=DEMO_OPTIONS,
    )

    # Render the index page.
    return render(request, "qr_code_demo/index.html", context=context)



"""Tests for qr_code application."""
import os

from django.test import SimpleTestCase
from pydantic import ValidationError

from qr_code.qrcode.constants import (
    DEFAULT_IMAGE_FORMAT,
    DEFAULT_MODULE_SIZE,
    DEFAULT_ERROR_CORRECTION,
    DEFAULT_VERSION,
    DEFAULT_ECI,
    DEFAULT_BOOST_ERROR,
    DEFAULT_ENCODING,
)
from qr_code.qrcode.serve import make_qr_code_url
from qr_code.qrcode.utils import QRCodeOptions
from qr_code.tests import TEST_TEXT, PNG_REF_SUFFIX, SVG_REF_SUFFIX

from qr_code.tests.utils import write_svg_content_to_file, get_resources_path, write_png_content_to_file


class TestApps(SimpleTestCase):
    def test_apps_attributes(self):
        from qr_code.apps import QrCodeConfig

        self.assertEqual(QrCodeConfig.name, "qr_code")
        self.assertEqual(QrCodeConfig.verbose_name, "Django QR Code")


class TestQRCodeOptions(SimpleTestCase):
    def test_qr_code_options(self):
        with self.assertRaises((TypeError, ValidationError)):
            QRCodeOptions(foo="bar")
        options = QRCodeOptions()
        self.assertEqual(options.border, 4)
        self.assertEqual(options.size, DEFAULT_MODULE_SIZE)
        self.assertEqual(options.image_format, DEFAULT_IMAGE_FORMAT)
        self.assertEqual(options.version, DEFAULT_VERSION)
        self.assertEqual(options.error_correction, DEFAULT_ERROR_CORRECTION)
        self.assertEqual(options.eci, DEFAULT_ECI)
        self.assertEqual(options.boost_error, DEFAULT_BOOST_ERROR)
        self.assertEqual(options.encoding.lower(), DEFAULT_ENCODING)
        options = QRCodeOptions(image_format="invalid-image-format")
        self.assertEqual(options.image_format, DEFAULT_IMAGE_FORMAT)

    def test_kw_save(self):
        options = QRCodeOptions(border=0, image_format="png", size=13)
        self.assertDictEqual(options.kw_save(), {"border": 0, "kind": "png", "scale": 13})


class TestWriteResourceData(SimpleTestCase):
    resource_file_base_name = "TestWriteResourceData"

    def test_write_svg(self):
        response = self.client.get(make_qr_code_url(TEST_TEXT))
        image_data = response.content.decode("utf-8")
        write_svg_content_to_file(TestWriteResourceData.resource_file_base_name, image_data)
        file_path_to_remove = os.path.join(get_resources_path(), TestWriteResourceData.resource_file_base_name + SVG_REF_SUFFIX)
        os.remove(file_path_to_remove)

    def test_write_png(self):
        response = self.client.get(make_qr_code_url(TEST_TEXT))
        image_data = response.content
        write_png_content_to_file(TestWriteResourceData.resource_file_base_name, image_data)
        file_path_to_remove = os.path.join(get_resources_path(), TestWriteResourceData.resource_file_base_name + PNG_REF_SUFFIX)
        os.remove(file_path_to_remove)
